# !/usr/bin/python

# Copyright (c) 2007 Randal Barlow
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import dbus

class NetBus:
    def __init__(self):
        self.dService = "org.freedesktop.NetworkManager"
        self.dObjectPath = "/org/freedesktop/NetworkManager"
        self.dInterface = "org.freedesktop.NetworkManager"
        self.deviceInt = "org.freedesktop.NetworkManager.Devices"
        self.deviceLocationCore = "/org/freedesktop/NetworkManager/Devices/"       
        self.session_bus = dbus.SystemBus()
        self.proxy_obj_NetworkManager = self.session_bus.get_object(self.dService, self.dObjectPath)
        self.dbusNetworkManager = dbus.Interface(self.proxy_obj_NetworkManager, self.dInterface)        
    ##
    ##
    def getActiveNetwork(self,device):
        """
        List the active network on a device
        
        If device is None it loads the module wide defualt set earlier in setDevice
        """
        if device == None:
            if 'device' in self.__dict__:    
                device = self.device
            else:pass
        if 'dbusDevice' not in self.__dict__:
            self.setDevice(self.getDevices()[0])    
            device = self.device
        if self.dbusDevice.getType() == 2:
            network_raw = self.dbusDevice.getActiveNetwork()
            network = network_raw.__str__().replace("/org/freedesktop/NetworkManager/Devices/" + device + "/Networks/","")
            return True, network
        else:
            return False, None
    def getNetworks(self,device):
        """
        List all networks on a device
        
        If device is None it loads the module wide defualt set earlier in setDevice
        """
        if device == None:
            if 'device' in self.__dict__:    
                device = self.device
        if 'dbusDevice' not in self.__dict__:
            self.setDevice(self.getDevices()[0])    
            device = self.device
        if self.dbusDevice.getType() == 2:
            networks_raw = self.dbusDevice.getNetworks()
            networks = []
            for item in networks_raw: networks.append(item.__str__().replace("/org/freedesktop/NetworkManager/Devices/" + device + "/Networks/",""))
            return True, networks
        else:
            return False, None
    ##
    ##
    def getDevices(self):
        """
        list all devices
        """
        devices = []
        for item in self.dbusNetworkManager.getDevices(): devices.append(item.__str__().replace("/org/freedesktop/NetworkManager/Devices/",""))
        return devices
    def setDevice(self,device):
        """
        sets a class wide device
        """
        deviceLocationFinal = self.deviceLocationCore + device
        proxy_obj_device = self.session_bus.get_object(self.dInterface, deviceLocationFinal)
        self.dbusDevice = dbus.Interface(proxy_obj_device, self.deviceInt)
        self.device = device
    ##
    ##
    def setActiveDevice(self,device,network):
        if device == None:
            if 'device' in self.__dict__:    
                device = self.device
        deviceLocationFinal = dbus.ObjectPath(self.deviceLocationCore + device)
        self.dbusNetworkManager.setActiveDevice(deviceLocationFinal,dbus.String(network))
    def createWirelessNetwork(self,device,network,security):
        if device == None:
            if 'device' in self.__dict__:    
                device = self.device
        deviceLocationFinal = dbus.ObjectPath(self.deviceLocationCore + device)
        self.dbusNetworkManager.createWirelessNetwork(deviceLocationFinal,dbus.String(network),security)
    ## 
    ##
    def getNetworkMap(self,network,device):
        """
        Retrieves a Networks dbus location Interface
        """
        if device == None:
            if 'device' in self.__dict__:    
                device = self.device
            else:device = self.getDevices()[0]
        if network == None: network = self.getActiveNetwork(device)[1]
        deviceLocationFinal = self.deviceLocationCore + device
        networkLocationFinal = deviceLocationFinal + '/Networks/' + network
        proxy_obj_device = self.session_bus.get_object(self.dInterface, networkLocationFinal)
        dbusNetworkTemp = dbus.Interface(proxy_obj_device, self.deviceInt)
        return dbusNetworkTemp
    def getNetworkExample(self,device,network):
        """
        A example call to get some call from a network
        
        working:
        getName
        getStrength
        getEncrypted
        getRate
        getFrequency
        """
        net = self.getNetworkMap(network,device)
        #retun some example call
        return net.getName()
    def getNetworkStrength(self,device,network):
        """
        A example call to get the strength of a a network
        """
        net = self.getNetworkMap(network,device)
        return net.getStrength()
    def getNetworkEncrypted(self,device,network):
        """
        A example call to the encryption status from a network
        """
        net = self.getNetworkMap(network,device)
        return net.getEncrypted()
